;
; Game concept for the Atari 2600 VCS
; Project name: SCSIcide
;
; Copyright 2001
; By Joe Grand, jgrand@mindspring.com (http://www.mindspring.com/~jgrand/atari)
; Last update: 6.May.2001
;
; dasm source.s -f3 -osource.bin to compile
;
; Tested with StellaX v1.1.3a
;
; DONE
;
; 1. Draw score, logo, initial playfield [initial playfield jg-0]
; 2. Enable movement up and down with joystick [joystick jg-1]
; 3. Display drive head on the right side of the screen
; 4. Draw playfield, with moving drive head [drivehead jg-2]
; 5. Added 6-digit score display routine with hexadecimal score
; 6. Added IncScore routine to add X (up to $FF) to the score,
;    handles rolling over. [hexscore jg-3]
; 7. Added data latency buffer (timer) [gauge_PF]
; 8. Added byte counter with color variation (matches score) [buffers jg-4]
; 9. Added data blobs, supports various speeds and horizontal position
;    (Atari 2600 MultiSprite Demo 1.1, Piero Cavina) [blobs]
; 10. Modified kernel to wrap-around data [wrap jg-5]
;
;
; REMOVED FROM PREVIOUS VERSION
;
; o Nothing
;
;
; NEXT STEPS
;
; o Basic gameplay: If data bit is read, increase score and byte counter
; o Set color of the required data bit to read
;
;
; POTENTIAL GAME FEATURES
;
; o Need to read correct color data bits in order to advance level
; o Bit size becomes smaller (fragmented) and faster as level increases
; o Latency buffer decreases when colored data bit missed
; o Bad blocks are any data bit read of the incorrect color
; o Once all bits are read, need to write contiguous bits back to a track
; o Paddle instead of joystick
; o Title screen
; o Demo/attract mode
; o Add sound
; o Easter egg :)
;

;
; Clear memory and registers
; Set up variables
; Loop:
;    Do the vertical blank
;    Do game calculations
;    Draw screen
;    Do more calculations during overscan
;    Wait for next vertical blank
; End Loop.
;

        processor 6502
        include vcs.h

        SEG.U defines

NUMGRP                  = 10    ; Number of graphics (player 1 data)
GRPHEIGHT               = 7     ; Height of each graphic

XMIN                    = 8
XMAX                    = 158

YTOP                    = 0     ; Top-most line of playfield (as drawn in kernel)
YBOTTOM                 = 72    ; Bottom-most line of playfield (two-line kernel)

HORPOS_P0               = 144   ; Horizontal position player 0 (drivehead) - fixed

COLOR_SCORE             = $B8   ; score/Time Remaining color (Green)
COLOR_DRIVEHEAD         = $18   ; Drive Head color (Yellow)
COLOR_GAUGE             = $34   ; Gas gauge_PF background color (Red)
COLOR_DATA              = $78   ; Default data color (Blue)
COLOR_LINE              = $F4   ; Line color (Light Brown)
COLOR_TRACK             = $F3   ; Track color (Brown)
COLOR_BG                = $00   ; Background color (Black)


        SEG.U vars
        org $80         ; Allocate memory, PIA has 128 bytes of RAM ($80-$FF). temp_stack from $FF

temp                    ds      1
temp_stack              ds      1
joystick0               ds      1       ; Player 0's joystick value

horPosP1                ds      NUMGRP  ; Horizontal position player 1 (data)
horPosP1_FC             ds      NUMGRP
horMotP1                ds      NUMGRP  ; Motion values player 1 (data) - 1 (slow) to 6 (fast)

verPosP0                ds      1       ; Vertical position player 0 (drivehead)

grpCount                ds      1       ; Group counter
grpY                    ds      NUMGRP
grpY_Next               ds      1       ; First line of next group
grpDistance             ds      1       ; Distance between player0 and top of group

coll                    ds      1       ; Collision flags
nocc                    ds      NUMGRP

currentDataColor        ds      1       ; Color of data to match

score                   ds      3       ; Player score (6 hex digits)
scoreCount              ds      1       ; Counter for drawing score
scorePtr0               ds      2       ; Pointer to score shapes
scorePtr1               ds      2
scorePtr2               ds      2
scorePtr3               ds      2
scorePtr4               ds      2
scorePtr5               ds      2

gauge_PF                ds      2       ; Data latency buffer, PF0 and PF1 (1st half of screen)
byteCount_PF            ds      2       ; Number of data bits read, PF1 and PF2 (2nd half of screen)

;gameNumber             ds      1       ; Current game variation being played


        SEG code
        ;org $F800       ; Limit ourselves to 2K like the old days
        org $F000       ; 4K

Start
;
; The 2600 powers up in a completely random state, except for the PC which
; is set to the location at $FFC.  Therefore the first thing we must do is
; to set everything up inside the 6502.
;

        SEI             ; Disable interrupts, if there are any.
        CLD             ; Clear BCD math bit.
        LDX  #$FF
        TXS             ; Set stack to beginning.

        ; Loop to clear memory and TIA registers
        LDA  #0
B1      STA  0,X
        DEX
        BNE  B1

        ; The above routine does not clear location 0, which is VSYNC

        JSR  GameInit           ; Initial variable and register setup
MainLoop
        JSR  VerticalBlank      ; Execute the vertical blank.
        JSR  CheckSwitches      ; Check console switches.
        JSR  GameCalc           ; Do calculations during VBLANK
        JSR  DrawScreen         ; Draw the screen
        JSR  OverScan           ; Do more calculations during overscan
        JMP  MainLoop           ; Continue forever.


VerticalBlank  ;*********************** VERTICAL BLANK HANDLER

;
; Beginning of the frame - at the end of overscan.
;

        LDA  #0
        STA  VBLANK     ; Disable video (helpful for PAL systems?).

        LDA  #2         ; VBLANK was set at the beginning of overscan.
        STA  WSYNC
        STA  WSYNC
        STA  WSYNC

        STA  VSYNC      ; Begin vertical sync

        STA  WSYNC      ; First line of VSYNC
        STA  WSYNC      ; Second line of VSYNC

        ; Set the timer to go off just before the end of vertical blank space
        ; 37 scanlines * 76 cycles = 2812 cycles / 64 = approx. 44
        LDA  #44
        STA  TIM64T

;
; End the VSYNC period.
;
        LDA  #0
        STA  WSYNC ; Third line of VSYNC.
        STA  VSYNC ; (0)
        RTS


CheckSwitches ;*************************** CONSOLE SWITCH HANDLER

        LDA  SWCHA      ; Read joystick 0
        STA  joystick0  ; Store for later use

        LDA  SWCHB      ; Read the switches
        LSR             ; Shift the Reset switch
        ROR             ; Rotate Reset to Negative flag and Carry has select

        ; PAL/NTSC select check

        RTS

;
; Minimal game calculations, just to get the ball rolling.
;
GameCalc ;******************************* GAME CALCULATION ROUTINES

        LDX  #$FF               ; Temporarily increment the score for demo purposes
        JSR  IncScore

;
; Move data objects horizontally, handle collisions (Courtesy of Piero Cavina PCMSD 1.1 source)
;

        LDX  #NUMGRP-1
MoveLP
        LDA  horPosP1,X         ; Get current position
        CLC
        ADC  horMotP1,X         ; Increment/decrement by motion value
        STA  horPosP1,X
        CMP  #XMIN              ; Are we at the minimum X value (left edge of screen)?
        BCC  SwapX0
        CMP  #XMAX              ; Are we at the maximum X value (right edge of screen)?
        BCS  SwapX
        JMP  MoveDone           ; We must be somewhere in between
SwapX0
        LDA  #XMIN              ; Minimum
        SEC
        SBC  horPosP1,X
        CLC
        ADC  #XMIN
        STA  horPosP1,X
SwapX                           ; Maximum
        LDA  #XMIN              ; Wrap around to the beginning of the track
        STA  horPosP1,X
MoveDone
        LDA  horPosP1,X         ; Convert X position to FC format
        JSR  Convert
        STA  horPosP1_FC,X

        ;
        ; Check for any collisions between P1 (data) and P0 (drivehead)
        ;
;       LDA    nocc
;       BNE    NOCL
;
;       LDA    coll,X
;       BPL    NOCL1
;
;       LDA    horMotP1,X
;       EOR    PLXMOT
;       BMI    DOSW
;       LDA    horMotP1,X
;       STA    PLXMOT
;       JMP    NOTSW
;
;DOSW:  LDA    PLXMOT
;       EOR    #$FF
;       CLC
;       ADC    #1
;       STA    PLXMOT
;
;
;NOTSW: LDA    #64
;       STA    nocc
;NOCL:  DEC    nocc
;NOCL1: LDA    #0
;       STA    coll,X

        DEX
        BPL  MoveLP

        ;
        ; Set new vertical position of drivehead based on joystick value
        ;
        LDA  joystick0          ; Load the joystick switches
        ROL
        ROL
        BMI  NotDown            ; Joystick pushed down?
        LDX  verPosP0
        CPX  #YBOTTOM
        BEQ  NoStick            ; If verPosP0 < YBOTTOM, don't change
        INC  verPosP0
        INC  verPosP0
        JSR  DecGauge           ; Decrease gas gauge
        LDA  #0                 ; Clear byte counter
        STA  byteCount_PF
        STA  byteCount_PF+1
        INC  currentDataColor
        JMP  NoStick
NotDown
        ROL
        BMI  NoStick            ; Joystick pushed up?
        LDX  verPosP0
        CPX  #YTOP
        BEQ  NoStick            ; If verPosP0 > YTOP, don't change
        DEC  verPosP0
        DEC  verPosP0
        JSR  IncGauge           ; Increase gas gauge
        JSR  IncCount           ; Increase byte counter
NoStick
        LDA  #0
        STA  grpCount           ; Initialize group counter
        STA  grpY               ; First line of first group
        LDA  #GRPHEIGHT+1
        STA  grpY_Next          ; First line of next (second) group
        LDA  verPosP0
        STA  grpDistance

        ;
        ; Set pointers to number data (based on current score)
        ; score score+1 score+2
        ;
        LDX  #3-1
        LDY  #10
ScoreLoop
        LDA  score,X
        AND  #$0F               ; Mask it
        ASL                     ; Multiply by 8
        ASL
        ASL

        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  scorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  scorePtr0+1,Y

        DEY
        DEY

        LDA  score,X
        AND  #$F0               ; Mask it
        LSR                     ; $00,$08,$10,$18,$20, ... ( / 8 = value)

        ADC  #<FontPage         ; Store the 2-byte pointer
        STA  scorePtr0,Y
        LDA  #0
        ADC  #>FontPage
        STA  scorePtr0+1,Y

        DEY
        DEY
        DEX
        BPL  ScoreLoop

        RTS


DrawScreen ;**************************** SCREEN DRAWING ROUTINES
;
; Initialize display variables.
;
        LDA  #COLOR_BG
        STA  COLUBK             ; Background will be black
        LDA  currentDataColor   ; Set the score color (P0 and P1)
        STA  COLUP0
        STA  COLUP1
        LDX  #3
        STX  NUSIZ0             ; Three copies of player, close together (for score)
        STX  NUSIZ1

WaitVBlank
        LDA  INTIM              ; Loop until timer is done - wait for the proper scanline
                                ; (i.e. somewhere in the last line of VBLANK)
        BNE  WaitVBlank         ; Whew! We're at the beginning of the frame
        LDA  #0
        STA  WSYNC              ; End the current scanline
        STA  HMOVE              ; Add horizontal motion to position sprites
        STA  VBLANK             ; End the VBLANK period with a zero, enable video.
        STA  WSYNC              ; Wait 2 scanlines before drawing score
        STA  WSYNC

        ;
        ; Horizontally position P0 and P1 for score
        ;
        LDX  #22
Pause1
        DEX
        BNE  Pause1
        STA  RESP0

        STA  WSYNC
        LDX  #22
Pause2
        DEX
        BNE  Pause2
        NOP
        STA  RESP1
        STA  HMCLR
        LDA  #%10100000         ; P1 position 2 less than P0
        STA  HMP1
        LDA  #%11000000
        STA  HMP0
        STA  WSYNC
        STA  HMOVE
        STA  WSYNC


ScanLoop ; ================================== SCANNING LOOP
;
; There are 228/3 cycles per line, so we have a maximum of 73 cycles
; after STA WSYNC
;
; Display the scores - 6 digit, courtesy of Robin Harbron
;
        LDA  #8-1              ; Lines to display
        STA  scoreCount

        LDX  #9
Pause3
        DEX
        BNE  Pause3
        NOP
        NOP
        NOP

        TSX
        STX  temp_stack         ; Save the stack pointer (Andrew Davie)
DrawScore
        LDY  scoreCount         ; 3
        LDA  (scorePtr0),Y      ; 5
        STA  GRP0               ; 3 = 11

        BIT  $00

        LDA  (scorePtr1),Y      ; 5
        STA  GRP1               ; 3
        LDA  (scorePtr5),Y      ; 5
        TAX                     ; 2
        TXS                     ; 2
        LDA  (scorePtr2),Y      ; 5
        STA  temp               ; 3
        LDA  (scorePtr3),Y      ; 5
        TAX                     ; 2
        LDA  (scorePtr4),Y      ; 5 = 37

        LDY  temp               ; 3
        STY  GRP0               ; 3
        STX  GRP1               ; 3
        STA  GRP0               ; 3
        TSX                     ; 2
        STX  GRP1               ; 3 = 17

        DEC  scoreCount         ; 5
        BPL  DrawScore          ; 3 =  8

        LDX  temp_stack
        TXS

        LDA  #0
        STA  GRP0
        STA  GRP1

        ;
        ; Skip down a few lines
        ;
        LDY  #4
DrawWait
        STA  WSYNC
        DEY
        BNE  DrawWait

        ; Position drivehead (horizontally)
        LDA  #HORPOS_P0         ; Convert Player0 X position to FC format
        JSR  Convert
        STA  WSYNC              ; Prepare to position Player0
        STA  HMP0               ; Remember, we're still doing VBLANK now
        AND  #$0F
        TAY
PosWait
        DEY
        BPL  PosWait
        STA  RESP0
        STA  WSYNC
        STA  HMOVE

        LDX  #P_Single
        STX  NUSIZ0             ; One copy of player

        LDX  #P_Single
        ;LDX  #P_Double
        ;LDX  #P_TwoClose
        STX  NUSIZ1

        ; Setup colors
        LDA  #COLOR_DRIVEHEAD
        STA  COLUP0
        LDA  #COLOR_DATA
        STA  COLUP1

        STA  HMCLR              ; Clear horizontal motion (of Player 0)

        LDA  #0                 ; Use playfield (black) to shorten length of track
        STA  COLUPF
        STA  PF1
        STA  PF2
        LDA  #$30
        STA  PF0
        LDA  #$05               ; Reflect,No score,High Priority,1 pixel-wide ball
        STA  CTRLPF

;
; Start drawing the actual disk playfield (10 lanes)
;
; We're going to draw #NUMGRP groups, each made of:
; 2 scanlines for Player1 positioning with Player0, plus
; #GRPHEIGHT*2 scanlines with Player1 and Player0.
;
; (Courtesy of Piero Cavina PCMSD 1.1 source)
;
Kernel
        LDA  grpDistance        ; Distance between Player0 <-> top of group
        CMP  #GRPHEIGHT+1       ; Is Player0 inside current group?
        BCC  DrawP0             ; Yes, we'll draw it...
        LDX  #0                 ; No, draw instead a blank sprite
        BEQ  BlankP0
DrawP0
        LDA  grpY_Next          ; We must draw Player0, and we'll start
        SEC                     ; from the (grpY_Next-verPosP0)th byte.
        SBC  verPosP0
        TAX                     ; Put the index to the first byte into X
BlankP0
        STX  temp               ; and remember it.

        LDY  grpCount           ; Store any collision between Player0 and
        LDA  CXPPMM             ; Player1 happened while drawing the
        ORA  coll,Y             ; last group.
        STA  coll,Y

        LDA  horPosP1_FC,Y      ; Get Player1 position
        LDY  HeadFrame,X        ; Get Player0 pattern

        LDX  #0
        STA  WSYNC              ; Start with a new scanline.
        STY  GRP0               ; Set Player0 pattern
        STA  HMP1               ; Prepare Player1 fine motion
        AND  #$0F               ; Prepare Player1 coarse positioning
        TAY
PosWait2
        DEY                     ; Waste time
        BPL  PosWait2
        STA  RESP1              ; Position Player1

        STA  WSYNC              ; Wait for next scanline
        STA  HMOVE              ; Apply fine motion

        LDA  #COLOR_LINE
        STA  COLUBK             ; Draw track seperator

;
; Now prepare various things for the next group
;
        LDA  grpY_Next          ; Update this group and next group
        STA  grpY               ; top line numbers
        CLC
        ADC  #GRPHEIGHT+1
        STA  grpY_Next

        LDA  verPosP0           ; Find out which 'slice'
        SEC                     ; of Player0 we'll have to draw.
        SBC  grpY               ; We need the distance of Player0
        BPL  DPos               ; from the top of the group.
        EOR  #$FF
        CLC
        ADC  #1                 ; A = ABS(verPosP0-grpY)
DPos
        STA  grpDistance

        LDX  temp               ; Pointer to the next byte of Player0
        INX                     ; pattern. Use X while drawing the group

        LDA  #0                 ; Clear collisions
        STA  CXCLR

        LDY  #GRPHEIGHT-1       ; Initialize line counter (going backwards)
DrawGrp
        LDA  #$0
        STA  WSYNC              ; Wait for a new line
        LDA  #COLOR_TRACK
        STA  COLUBK             ; Set background color
        LDA  HeadFrame,X
        STA  GRP0               ; Set Player0 shape
        LDA  DataFrame,Y
        STA  GRP1               ; Set Player1 shape

        STA  WSYNC              ; Wait for a new scanline
        INX                     ; Update the index to next byte of Player0
        DEY                     ; Decrement line counter
        BPL  DrawGrp            ; Go on with this group if needed

        INC  grpCount           ; Increment current group number
        LDA  grpCount           ;
        CMP  #NUMGRP            ; Is there another group to do?
        BCS  OuterKernel        ; No, exit
        JMP  Kernel             ; Yes, go back. (Using JMP because a branch
                                ; would be out of range)
OuterKernel
        STA  WSYNC              ; Finish current scanline

        LDA  #0                 ; Avoid bleeding
        STA  GRP0
        STA  GRP1
        LDA  #COLOR_LINE        ; Draw bottom track seperator
        STA  COLUBK
        STA  WSYNC

        LDA  #COLOR_BG          ; Black line
        STA  COLUBK
        LDY  #8
DrawWait2
        STA  WSYNC
        DEY
        BNE  DrawWait2

;
; Draw gas gauge_PF/data latency and byte counter buffer
;
        LDA  #$30
        STA  CTRLPF             ; No Reflect,No score,Low Priority,8 pixel-wide ball

        LDX  #$07               ; Number of lines alike
NxtLine
        STA  WSYNC              ; Wait for line to finish

        LDA  #COLOR_SCORE       ; [0] +4 green
        STA  COLUPF             ; [4] +3
        LDA  #COLOR_GAUGE       ; [7] +4 red
        STA  COLUBK             ; [11] +3 set background

        ; First half of screen
        LDA  gauge_PF           ; [14] +3
        STA  PF0                ; [17] +3
        LDA  gauge_PF+1         ; [20] +3
        STA  PF1                ; [23] +3
        LDA  #0                 ; [26] +3
        STA  PF2                ; [29] +3

        NOP                     ; [32] +2
        NOP                     ; [34] +2
        NOP                     ; [36] +2
        NOP                     ; [38] +2

        STA  COLUBK             ; [40] +3 end of gauge -> background color back to black
        STA  PF0                ; [43] +3

        ; Second half of screen - display number of bytes currently retrieved
        LDA  currentDataColor   ; [46] +4
        STA  COLUPF             ; [50] +3
        LDA  byteCount_PF       ; [53] +4
        STA  PF1                ; [57] +3
        LDA  byteCount_PF+1     ; [60] +4
        STA  PF2                ; [64] +3

        DEX                     ; [67] +5
        BNE  NxtLine            ; [72] +3 (take branch)

;
; Clear all registers here to prevent any possible bleeding.
;
        LDA  #2
        STA  WSYNC              ; Finish this scanline.
        STA  VBLANK             ; Make TIA output invisible,
        ; Now we need to worry about it bleeding when we turn
        ; the TIA output back on.
        LDY  #0
        STY  PF0
        STY  PF1
        STY  PF2
        RTS


OverScan   ;***************************** OVERSCAN CALCULATIONS
;
; For the Overscan routine, one might take the time to process such
; things as collisions.
;
        LDX  #30
KillLines
        STA  WSYNC
        DEX
        BNE  KillLines
        RTS


GameInit   ;***************************** INITIAL VARIABLE & REGISTER SETUP
;
; GameInit could conceivably be called when the Select key is pressed,
; or some other event.
;
        LDA  #COLOR_SCORE
        STA  currentDataColor   ; Default to COLOR_SCORE

        ;
        ; Clear data pointers
        ;
        LDX  #11
        LDA  #0
Clear2
        STA  scorePtr0,X
        DEX
        BPL  Clear2

        STA  WSYNC
        LDY  #3
Pos0
        DEY
        BNE  Pos0
        STA  RESP0

        ; score
        STA  score      ; A = 0
        STA  score+1
        STA  score+2

        ; Byte counter
        STA  byteCount_PF
        STA  byteCount_PF+1

        ; Gas gauge
        LDA  #$F0
        STA  gauge_PF
        LDA  #$FF
        STA  gauge_PF+1

        ;
        ; Player 0 (drivehead) initialization
        ;
        LDA  #36                ; Start positioned in center
        STA  verPosP0

;
; Initialize objects positions and motion
;
        LDX  #NUMGRP-1          ; 10 lanes
DataInit
        ;
        ; In reality, these configurations will be set by the game selection
        ; Currently, it is for demo purposes only
        ;
        LDA  #0
        STA  horPosP1,X         ; Diagonal
        DEX
        BPL  DataInit

        LDA  #1
        STA  horMotP1
        STA  horMotP1+9
        LDA  #2
        STA  horMotP1+1
        STA  horMotP1+8
        LDA  #3
        STA  horMotP1+2
        STA  horMotP1+7
        LDA  #4
        STA  horMotP1+3
        STA  horMotP1+6
        LDA  #5
        STA  horMotP1+4
        LDA  #6
        STA  horMotP1+5

        RTS

;
; Increase the score by X (up to $FF)
; Handles roll of score from 0000FF to 000100, etc.
;
IncScore
        CLC             ; Clear carry
        TXA             ; score increment into accumulator
        LDX  #3-1
Loop
        ADC  score,X    ; Add LSB
        STA  score,X
        LDA  #0         ; MSB 2 bytes = 0
        DEX
        BPL  Loop
        RTS

;
; Increase/decrease the value of the data latency buffer by 1 pixel
;
IncGauge
        LDA  gauge_PF
        CMP  #$F0
        BEQ  IncPF1
        ASL
        ORA  #$10
        STA  gauge_PF
        JMP  IncDone
IncPF1
        LDA  gauge_PF+1
        LSR
        ORA  #$80
        STA  gauge_PF+1
IncDone
        RTS


DecGauge
        LDA  gauge_PF+1
        CMP  #$0
        BEQ  DecPF0
        ASL
        STA  gauge_PF+1
        JMP  DecDone
DecPF0
        LDA  gauge_PF
        LSR
        AND  #$F0
        STA  gauge_PF
DecDone
        RTS


;
; Increase the value of the byte counter by 2 pixels
;
IncCount
        LDA  byteCount_PF
        CMP  #$FF
        BEQ  IncPF2
        LSR
        LSR
        ORA  #$C0
        STA  byteCount_PF
        JMP  IncDone2
IncPF2
        LDA  byteCount_PF+1
        ASL
        ASL
        ORA  #$03
        STA  byteCount_PF+1
IncDone2
        RTS


;
; Straight from "Air-Sea Battle", here's the routine
; to convert from standard X positions to FC positions
;
Convert
        STA  temp
        BPL  LF34B
        CMP  #$9E
        BCC  LF34B
        LDA  #$00
        STA  temp
LF34B
        LSR
        LSR
        LSR
        LSR
        TAY
        LDA  temp
        AND  #$0F
        STY  temp
        CLC
        ADC  temp
        CMP  #$0F
        BCC  LF360
        SBC  #$0F
        INY
LF360
        CMP  #$08
        EOR  #$0F
        BCS  LF369
        ADC  #$01
        DEY
LF369
        INY
        ASL
        ASL
        ASL
        ASL
        STA  temp
        TYA
        ORA  temp
        RTS



HeadFrame
                ; Note the leading and trailing $00's
                .byte $00, $00, $00, $00, $00, $00, $00, $00, $00

                .byte   %00011110 ; |   XXXX |
                .byte   %00111011 ; |  XXX XX|
                .byte   %00110000 ; |  XX    |
                .byte   %11111000 ; |XXXXX   |
                .byte   %00110000 ; |  XX    |
                .byte   %00111011 ; |  XXX XX|
                .byte   %00011110 ; |   XXXX |

                .byte $00, $00, $00, $00, $00, $00, $00


DataFrame
                 .byte   %00000000 ; |        |
                 .byte   %11111111 ; |XXXXXXXX|
                 .byte   %11111111 ; |XXXXXXXX|
                 .byte   %11111111 ; |XXXXXXXX|
                 .byte   %11111111 ; |XXXXXXXX|
                 .byte   %11111111 ; |XXXXXXXX|
                 .byte   %00000000 ; |        |

                 ; DataFrame & $3F
                 ;.byte   %00000000 ; |        |
                 ;.byte   %00111111 ; |  XXXXXX|
                 ;.byte   %00111111 ; |  XXXXXX|
                 ;.byte   %00111111 ; |  XXXXXX|
                 ;.byte   %00111111 ; |  XXXXXX|
                 ;.byte   %00111111 ; |  XXXXXX|
                 ;.byte   %00000000 ; |        |

                 ; DataFrame & $0F
                 ;.byte   %00000000 ; |        |
                 ;.byte   %00001111 ; |    XXXX|
                 ;.byte   %00001111 ; |    XXXX|
                 ;.byte   %00001111 ; |    XXXX|
                 ;.byte   %00001111 ; |    XXXX|
                 ;.byte   %00001111 ; |    XXXX|
                 ;.byte   %00000000 ; |        |

                 ; DataFrame & $03
                 ;.byte   %00000000 ; |        |
                 ;.byte   %00000011 ; |      XX|
                 ;.byte   %00000011 ; |      XX|
                 ;.byte   %00000011 ; |      XX|
                 ;.byte   %00000011 ; |      XX|
                 ;.byte   %00000011 ; |      XX|
                 ;.byte   %00000000 ; |        |


; All shapes are upside down to allow decrementing by Y as both
; a counter and a shape index
;
        org $FE00 ; *********************** GRAPHICS DATA

;
; Numeric font
;
FontPage
Zero
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

One
                .byte   %00111100 ; |  XXXX  |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00111000 ; |  XXX   |
                .byte   %00011000 ; |   XX   |

Two
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Three
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000110 ; | X   XX |
                .byte   %00111100 ; |  XXXX  |

Four
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %00001100 ; |    XX  |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01001100 ; | X  XX  |
                .byte   %00101100 ; |  X XX  |
                .byte   %00011100 ; |   XXX  |
                .byte   %00001100 ; |    XX  |

Five
                .byte   %01111100 ; | XXXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00000110 ; |     XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |

Six
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100000 ; | XX     |
                .byte   %01100010 ; | XX   X |
                .byte   %00111100 ; |  XXXX  |

Seven
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00011000 ; |   XX   |
                .byte   %00001100 ; |    XX  |
                .byte   %00000110 ; |     XX |
                .byte   %01000010 ; | X    X |
                .byte   %01111110 ; | XXXXXX |

Eight
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

Nine
                .byte   %00111100 ; |  XXXX  |
                .byte   %01000110 ; | X   XX |
                .byte   %00000110 ; |     XX |
                .byte   %00111110 ; |  XXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

A
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

B
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |

C
                .byte   %00111100 ; |  XXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100110 ; | XX  XX |
                .byte   %00111100 ; |  XXXX  |

D
                .byte   %01111100 ; | XXXXX  |
                .byte   %01100110 ; | XX  XX |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100010 ; | XX   X |
                .byte   %01100110 ; | XX  XX |
                .byte   %01111100 ; | XXXXX  |

E
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |

F
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |
                .byte   %01100000 ; | XX     |
                .byte   %01100000 ; | XX     |
                .byte   %01111110 ; | XXXXXX |


        .byte "J. Grand"

;
; Avoid $FFF8 to allow use with Supercharger
;

;
; Set up the 6502 interrupt vector table
;
        org $FFFC

        .word Start     ; Reset
        .word Start     ; IRQ

        END